@@ -1,12 +1,12 @@
-use crate::error::{IntErr, Interrupt, Never};
+use crate::error::{FendError, Interrupt};
 use crate::num::real::{self, Real};
 use crate::num::Exact;
-use crate::num::{Base, ConvertToUsizeError, DivideByZero, FormattingStyle};
+use crate::num::{Base, FormattingStyle};
 use std::cmp::Ordering;
 use std::fmt;
 use std::ops::Neg;
 
-#[derive(Clone, PartialEq, Eq)]
+#[derive(Clone, PartialEq, Eq, Hash)]
 pub(crate) struct Complex {
     real: Real,
     imag: Real,
@@ -30,12 +30,9 @@ pub(crate) enum UseParentheses {
 }
 
 impl Complex {
-    pub(crate) fn try_as_usize<I: Interrupt>(
-        self,
-        int: &I,
-    ) -> Result<usize, IntErr<ConvertToUsizeError, I>> {
+    pub(crate) fn try_as_usize<I: Interrupt>(self, int: &I) -> Result<usize, FendError> {
         if self.imag != 0.into() {
-            return Err(ConvertToUsizeError::ComplexNumber.into());
+            return Err(FendError::ComplexToInteger);
         }
         self.real.try_as_usize(int)
     }
@@ -47,7 +44,7 @@ impl Complex {
         }
     }
 
-    pub(crate) fn factorial<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
+    pub(crate) fn factorial<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
         if self.imag != 0.into() {
             return Err("factorial is not supported for complex numbers"
                 .to_string()
@@ -59,11 +56,7 @@ impl Complex {
         })
     }
 
-    pub(crate) fn pow<I: Interrupt>(
-        self,
-        rhs: Self,
-        int: &I,
-    ) -> Result<Exact<Self>, IntErr<String, I>> {
+    pub(crate) fn pow<I: Interrupt>(self, rhs: Self, int: &I) -> Result<Exact<Self>, FendError> {
         if self.imag != 0.into() || rhs.imag != 0.into() {
             return Err(
                 "exponentiation is currently unsupported for complex numbers"
@@ -95,7 +88,7 @@ impl Complex {
         }
     }
 
-    pub(crate) fn abs<I: Interrupt>(self, int: &I) -> Result<Exact<Self>, IntErr<String, I>> {
+    pub(crate) fn abs<I: Interrupt>(self, int: &I) -> Result<Exact<Self>, FendError> {
         Ok(if self.imag.is_zero() {
             if self.real < 0.into() {
                 Exact::new(
@@ -146,7 +139,7 @@ impl Complex {
         base: Base,
         use_parentheses: UseParentheses,
         int: &I,
-    ) -> Result<Exact<Formatted>, IntErr<Never, I>> {
+    ) -> Result<Exact<Formatted>, FendError> {
         let style = if !exact && style == FormattingStyle::Auto {
             FormattingStyle::DecimalPlaces(10)
         } else if self.imag != 0.into() && style == FormattingStyle::Auto {
@@ -208,11 +201,7 @@ impl Complex {
         })
     }
 
-    pub(crate) fn root_n<I: Interrupt>(
-        self,
-        n: &Self,
-        int: &I,
-    ) -> Result<Exact<Self>, IntErr<String, I>> {
+    pub(crate) fn root_n<I: Interrupt>(self, n: &Self, int: &I) -> Result<Exact<Self>, FendError> {
         if self.imag != 0.into() || n.imag != 0.into() {
             return Err("roots are currently unsupported for complex numbers"
                 .to_string()
@@ -228,7 +217,7 @@ impl Complex {
         ))
     }
 
-    fn expect_real<I: Interrupt>(self) -> Result<Real, IntErr<String, I>> {
+    fn expect_real(self) -> Result<Real, FendError> {
         if self.imag.is_zero() {
             Ok(self.real)
         } else {
@@ -236,16 +225,14 @@ impl Complex {
         }
     }
 
-    pub(crate) fn sin<I: Interrupt>(self, int: &I) -> Result<Exact<Self>, IntErr<String, I>> {
+    pub(crate) fn sin<I: Interrupt>(self, int: &I) -> Result<Exact<Self>, FendError> {
         Ok(self.expect_real()?.sin(int)?.apply(Self::from))
     }
 
-    pub(crate) fn cos<I: Interrupt>(self, int: &I) -> Result<Exact<Self>, IntErr<String, I>> {
+    pub(crate) fn cos<I: Interrupt>(self, int: &I) -> Result<Exact<Self>, FendError> {
         // cos(self) == sin(pi/2 - self)
         let pi = Exact::new(Self::pi(), true);
-        let half_pi = pi
-            .div(Exact::new(2.into(), true), int)
-            .map_err(IntErr::into_string)?;
+        let half_pi = pi.div(Exact::new(2.into(), true), int)?;
         let sin_arg = half_pi.add(-Exact::new(self, true), int)?;
         Ok(sin_arg
             .value
@@ -255,89 +242,65 @@ impl Complex {
             .apply(Self::from))
     }
 
-    pub(crate) fn tan<I: Interrupt>(self, int: &I) -> Result<Exact<Self>, IntErr<String, I>> {
+    pub(crate) fn tan<I: Interrupt>(self, int: &I) -> Result<Exact<Self>, FendError> {
         let num = self.clone().sin(int)?;
         let den = self.cos(int)?;
-        num.div(den, int).map_err(IntErr::into_string)
+        num.div(den, int)
     }
 
-    pub(crate) fn asin<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
-        Ok(Self::from(
-            self.expect_real()?.asin(int).map_err(IntErr::into_string)?,
-        ))
+    pub(crate) fn asin<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
+        Ok(Self::from(self.expect_real()?.asin(int)?))
     }
 
-    pub(crate) fn acos<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
-        Ok(Self::from(
-            self.expect_real()?.acos(int).map_err(IntErr::into_string)?,
-        ))
+    pub(crate) fn acos<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
+        Ok(Self::from(self.expect_real()?.acos(int)?))
     }
 
-    pub(crate) fn atan<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
+    pub(crate) fn atan<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
         Ok(Self::from(self.expect_real()?.atan(int)?))
     }
 
-    pub(crate) fn sinh<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
+    pub(crate) fn sinh<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
         Ok(Self::from(self.expect_real()?.sinh(int)?))
     }
 
-    pub(crate) fn cosh<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
+    pub(crate) fn cosh<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
         Ok(Self::from(self.expect_real()?.cosh(int)?))
     }
 
-    pub(crate) fn tanh<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
+    pub(crate) fn tanh<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
         Ok(Self::from(self.expect_real()?.tanh(int)?))
     }
 
-    pub(crate) fn asinh<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
+    pub(crate) fn asinh<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
         Ok(Self::from(self.expect_real()?.asinh(int)?))
     }
 
-    pub(crate) fn acosh<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
-        Ok(Self::from(
-            self.expect_real()?
-                .acosh(int)
-                .map_err(IntErr::into_string)?,
-        ))
+    pub(crate) fn acosh<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
+        Ok(Self::from(self.expect_real()?.acosh(int)?))
     }
 
-    pub(crate) fn atanh<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
-        Ok(Self::from(
-            self.expect_real()?
-                .atanh(int)
-                .map_err(IntErr::into_string)?,
-        ))
+    pub(crate) fn atanh<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
+        Ok(Self::from(self.expect_real()?.atanh(int)?))
     }
 
-    pub(crate) fn ln<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
-        Ok(Self::from(
-            self.expect_real()?.ln(int).map_err(IntErr::into_string)?,
-        ))
+    pub(crate) fn ln<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
+        Ok(Self::from(self.expect_real()?.ln(int)?))
     }
 
-    pub(crate) fn log2<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
-        Ok(Self::from(
-            self.expect_real()?.log2(int).map_err(IntErr::into_string)?,
-        ))
+    pub(crate) fn log2<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
+        Ok(Self::from(self.expect_real()?.log2(int)?))
     }
 
-    pub(crate) fn log10<I: Interrupt>(self, int: &I) -> Result<Self, IntErr<String, I>> {
-        Ok(Self::from(
-            self.expect_real()?
-                .log10(int)
-                .map_err(IntErr::into_string)?,
-        ))
+    pub(crate) fn log10<I: Interrupt>(self, int: &I) -> Result<Self, FendError> {
+        Ok(Self::from(self.expect_real()?.log10(int)?))
     }
 
     pub(crate) fn is_definitely_one(&self) -> bool {
         self.real.is_definitely_one() && self.imag.is_definitely_zero()
     }
 
-    pub(crate) fn modulo<I: Interrupt>(
-        self,
-        rhs: Self,
-        int: &I,
-    ) -> Result<Self, IntErr<String, I>> {
+    pub(crate) fn modulo<I: Interrupt>(self, rhs: Self, int: &I) -> Result<Self, FendError> {
         Ok(Self::from(
             self.expect_real()?.modulo(rhs.expect_real()?, int)?,
         ))
@@ -346,7 +309,7 @@ impl Complex {
 
 #[allow(clippy::use_self)]
 impl Exact<Complex> {
-    pub(crate) fn add<I: Interrupt>(self, rhs: Self, int: &I) -> Result<Self, IntErr<Never, I>> {
+    pub(crate) fn add<I: Interrupt>(self, rhs: Self, int: &I) -> Result<Self, FendError> {
         let (self_real, self_imag) = self.apply(|x| (x.real, x.imag)).pair();
         let (rhs_real, rhs_imag) = rhs.apply(|x| (x.real, x.imag)).pair();
         let real = self_real.add(rhs_real, int)?;
@@ -360,7 +323,7 @@ impl Exact<Complex> {
         ))
     }
 
-    pub(crate) fn mul<I: Interrupt>(self, rhs: &Self, int: &I) -> Result<Self, IntErr<Never, I>> {
+    pub(crate) fn mul<I: Interrupt>(self, rhs: &Self, int: &I) -> Result<Self, FendError> {
         // (a + bi) * (c + di)
         //     => ac + bci + adi - bd
         //     => (ac - bd) + (bc + ad)i
@@ -382,11 +345,7 @@ impl Exact<Complex> {
         ))
     }
 
-    pub(crate) fn div<I: Interrupt>(
-        self,
-        rhs: Self,
-        int: &I,
-    ) -> Result<Self, IntErr<DivideByZero, I>> {
+    pub(crate) fn div<I: Interrupt>(self, rhs: Self, int: &I) -> Result<Self, FendError> {
         // (u + vi) / (x + yi) = (1/(x^2 + y^2)) * ((ux + vy) + (vx - uy)i)
         let (u, v) = self.apply(|x| (x.real, x.imag)).pair();
         let (x, y) = rhs.apply(|x| (x.real, x.imag)).pair();
