@@ -36,6 +36,10 @@ pub(crate) trait ValueTrait: fmt::Debug + BoxClone + 'static {
         None
     }
 
+    fn add(&self, _rhs: Value) -> Result<Value, FendError> {
+        Err(FendError::ExpectedANumber)
+    }
+
     fn is_unit(&self) -> bool {
         false
     }
@@ -184,6 +188,13 @@ impl Value {
         }
     }
 
+    pub(crate) fn add_dyn(self, rhs: Self) -> Result<Self, FendError> {
+        match self {
+            Self::Dynamic(d) => d.add(rhs),
+            _ => Err(FendError::ExpectedANumber),
+        }
+    }
+
     pub(crate) fn handle_num(
         self,
         eval_fn: impl FnOnce(Number) -> Result<Number, FendError>,
